#!/usr/bin/python3

import gettext
import os
import subprocess
import sys

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

import mintcommon.aptdaemon

# i18n
gettext.install("mintcommon", "/usr/share/locale")

class MintRemoveWindow:

    def __init__(self, desktopFile):
        self.desktopFile = desktopFile
        process = subprocess.run(["dpkg", "-S", self.desktopFile], stdout=subprocess.PIPE, encoding="utf-8")
        output = process.stdout
        package = output[:output.find(":")].split(",")[0]
        if process.returncode != 0:
            if not self.try_remove_flatpak(desktopFile):
                warnDlg = Gtk.MessageDialog(
                    modal=True,
                    destroy_with_parent=True,
                    message_type=Gtk.MessageType.WARNING,
                    buttons=Gtk.ButtonsType.YES_NO,
                    title=_("Uninstall"),
                    text=_("This menu item is not associated to any package. "
                           "Do you want to remove it from the menu anyway?"))
                warnDlg.set_property("skip-taskbar-hint", False)
                warnDlg.set_icon_name("system-software-install")
                warnDlg.get_widget_for_response(Gtk.ResponseType.YES).grab_focus()
                warnDlg.get_message_area().set_spacing(10)
                response = warnDlg.run()
                if response == Gtk.ResponseType.YES:
                    print(f"removing '{self.desktopFile}'")
                    for filename in (self.desktopFile, f"{self.desktopFile}.desktop"):
                        try:
                            os.unlink(filename)
                        except:
                            pass
                warnDlg.destroy()
            sys.exit(0)

        warnDlg = Gtk.MessageDialog(
                    modal=True,
                    destroy_with_parent=True,
                    message_type=Gtk.MessageType.WARNING,
                    buttons=Gtk.ButtonsType.OK_CANCEL,
                    title=_("Uninstall"),
                    text=_("The following packages will be removed:"))
        warnDlg.set_property("skip-taskbar-hint", False)
        warnDlg.set_icon_name("system-software-install")
        warnDlg.get_widget_for_response(Gtk.ResponseType.OK).grab_focus()
        warnDlg.get_message_area().set_spacing(10)

        treeview = Gtk.TreeView()
        column1 = Gtk.TreeViewColumn(_("Packages to be removed"))
        renderer = Gtk.CellRendererText()
        column1.pack_start(renderer, False)
        column1.add_attribute(renderer, "text", 0)
        treeview.append_column(column1)

        packages = []
        model = Gtk.ListStore(str)
        dependenciesString = subprocess.run(["apt-get", "-sq", "remove", package],
            stdout=subprocess.PIPE, encoding="utf-8").stdout
        dependencies = dependenciesString.split("\n")
        for dependency in dependencies:
            if dependency.startswith("Remv "):
                dependency = dependency.split()
                try:
                    model.append([" ".join(dependency[1:])])
                    packages.append(dependency[1])
                except:
                    print("Error parsing: ", dependency)
        treeview.set_model(model)
        treeview.show()

        scrolledwindow = Gtk.ScrolledWindow()
        scrolledwindow.set_shadow_type(Gtk.ShadowType.ETCHED_OUT)
        scrolledwindow.set_size_request(150, 150)
        scrolledwindow.add(treeview)
        scrolledwindow.show()

        warnDlg.get_content_area().add(scrolledwindow)

        self.apt = mintcommon.aptdaemon.APT(warnDlg)

        response = warnDlg.run()
        if response == Gtk.ResponseType.OK:
            self.apt.set_finished_callback(self.on_finished)
            self.apt.remove_packages(packages)
        warnDlg.destroy()

    def try_remove_flatpak(self, desktopFile):
        if not "flatpak" in desktopFile:
            return False

        if not os.path.exists('/usr/bin/mintinstall-remove-app'):
            return False

        flatpak_remover = subprocess.Popen(['/usr/bin/mintinstall-remove-app', desktopFile])
        retcode = flatpak_remover.wait()

        return retcode == 0

    def on_finished(self, _transaction=None, _exit_state=None):
        sys.exit(0)

if __name__ == "__main__":
    # Exit if the given path does not exist
    if len(sys.argv) < 2 or not os.path.exists(sys.argv[1]):
        sys.exit(1)

    for arg in sys.argv[1:]:
        arg = arg.split("=", 1)
        if arg[0] in ("HOME", "DISPLAY", "XAUTHORITY"):
            os.environ[arg[0]] = arg[1]

    mainwin = MintRemoveWindow(sys.argv[1])
    Gtk.main()
