def localized_ui(ui_file, gettext):
    """ Returns localized contents of Gtk.Builder XML UI definition

    Parameters:
        ui_file: Path to file containing Gtk.Builder XML UI definition
        gettext: Callback to gettext.gettext()

    Workaround for Gtk.Builder/GLib.dgettext creating encoding issues in
    non-UTF-8 environments. It translates all translatable strings in `ui_file`
    via the `gettext` callback before you pass it to Gtk.Builder.

    Usage:
        builder = Gtk.Builder.new_from_string(localized_ui(ui_file, _), -1)
    """
    from xml.etree import ElementTree
    try:
        tree = ElementTree.parse(ui_file)
        for node in tree.iter():
            if "translatable" in node.attrib:
                del node.attrib["translatable"]
                node.text = gettext(node.text)
        return ElementTree.tostring(tree.getroot(), encoding="unicode", method="xml")
    except:
        return open(ui_file).read()
