""" Module to work with GNOME 3 proxy settings via GProxyResolver and GSettings API """

from gi.repository import Gio

# GSettings API:

schema = "org.gnome.system.proxy"

def get_proxysettings():
    if schema in Gio.Settings.list_schemas():
        def get_keys(settings):
            _dict = {}
            for key in settings.list_keys():
                _dict[key] = settings.get_value(key).unpack()
            return _dict

        settings = Gio.Settings(schema)
        proxysettings = get_keys(settings)
        for child in settings.list_children():
            proxysettings[child] = get_keys(settings.get_child(child))
        return proxysettings
    return {}

def get_proxy_mode():
    if schema in Gio.Settings.list_schemas():
        settings = Gio.Settings(schema)
        if "mode" in settings.list_keys():
            return settings.get_value("mode").unpack()
    return None

def get_ignored_hosts():
    if schema in Gio.Settings.list_schemas():
        settings = Gio.Settings(schema)
        if "ignore-hosts" in settings.list_keys():
            return settings.get_value("ignore-hosts").unpack()
    return None

def proxies_enabled():
    mode = get_proxy_mode()
    return bool(mode and mode != "none")

# GProxyResolver API:

resolver = Gio.proxy_resolver_get_default()

def lookup(uri):
    if not "://" in uri:
        uri += "://"
    return resolver.lookup(uri)

# Helpers for other libraries:

def requests_proxies():
    """ proxies dictionary for requests library """
    proxies = {}
    if proxies_enabled():
        for protocol in ["http://", "https://"]:
            url = lookup(protocol)
            if url and url[0] != "direct://":
                proxies[protocol] = url[0]
    return proxies

def urllib_proxies():
    """ emulate urllib.request.getproxies() output """
    proxies = {}
    if proxies_enabled():
        for protocol in ["http", "https", "ftp", "socks"]:
            url = lookup(protocol)
            if url and url[0] != "direct://":
                proxies[protocol if protocol != "socks" else "all"] = url[0]
        exclusions = get_ignored_hosts()
        if exclusions:
            proxies["no"] = ",".join(exclusions)
    return proxies

def set_environment_variables():
    """ sets the xxx_proxy environment variables """
    import os
    for item in urllib_proxies().items():
        os.environ[f"{item[0]}_proxy"] = item[1]
